package com.isu.gaswellwatch.vo.command.etc;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import jakarta.validation.constraints.NotBlank;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/27 22:56
 */
@Getter
@Setter
@SuperBuilder
public class PlungerTimer extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = 888739283997908251L;

    /**
     * 柱塞上升时间。[0-999]:[0-59]:[0-59]
     */
    @NotBlank(message = "柱塞上升时间不能为空")
    private String plungerRiseTime;

    public PlungerTimer() {
        this.setCode("ETC.PLUNGER_TIMER");
    }

    @Override
    public boolean validate() {
        return super.validate() && StringUtils.isNotBlank(this.plungerRiseTime);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(60);
        //                                0003*2->hex
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     006A    0003     06
        // todo 使用功能码 10 修改多位, 点表中参考功能码为16
        // todo SensorDelay 类中同样使用功能码 10 修改多位, 待测
        command.append("0110006A000306").append(this.toHexString(this.plungerRiseTime));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setPlungerRiseTime(String plungerRiseTime) {
        this.plungerRiseTime = this.timingValidate(plungerRiseTime, "plungerRiseTime");
    }

}
